@@ -29,6 +29,14 @@
 /*
  * GVfsAfpName
  */
+struct _GVfsAfpName
+{
+  guint32 text_encoding;
+  gchar *str;
+  gsize len;
+
+  gint ref_count;
+};
 
 static void
 _g_vfs_afp_name_free (GVfsAfpName *afp_name)
@@ -503,6 +511,46 @@ g_vfs_afp_command_put_afp_name (GVfsAfpCommand *comm, GVfsAfpName *afp_name)
   }
 }
 
+static GVfsAfpName *
+filename_to_afp_pathname (const char *filename)
+{
+  gsize len;
+  char *str;
+  gint i;
+
+  while (*filename == '/')
+    filename++;
+  
+  len = strlen (filename);
+  
+  str = g_malloc (len); 
+
+  for (i = 0; i < len; i++)
+  {
+    if (filename[i] == '/')
+      str[i] = '\0';
+    else
+      str[i] = filename[i];
+  }
+  
+
+  return g_vfs_afp_name_new (0x08000103, str, len);
+}
+
+void
+g_vfs_afp_command_put_pathname (GVfsAfpCommand *comm, const char *filename)
+{
+  GVfsAfpName *pathname;
+  
+  /* PathType */
+  g_vfs_afp_command_put_byte (comm, AFP_PATH_TYPE_UTF8_NAME);
+
+  /* Pathname */
+  pathname = filename_to_afp_pathname (filename);
+  g_vfs_afp_command_put_afp_name (comm, pathname);
+  g_vfs_afp_name_unref (pathname);
+}
+
 void
 g_vfs_afp_command_pad_to_even (GVfsAfpCommand *comm)
 { 
@@ -602,17 +650,6 @@ typedef enum
   DSI_ATTENTION     = 8
 } DsiCommand;
 
-static void read_reply (GVfsAfpConnection *afp_connection);
-static void send_request (GVfsAfpConnection *afp_connection);
-
-static guint16
-get_request_id (GVfsAfpConnection *afp_connection)
-{
-  GVfsAfpConnectionPrivate *priv = afp_connection->priv;
-
-  return priv->request_id++;
-}
-
 typedef enum
 {
   REQUEST_TYPE_COMMAND,
@@ -642,6 +679,73 @@ free_request_data (RequestData *req_data)
   g_slice_free (RequestData, req_data);
 }
 
+static void
+g_vfs_afp_connection_init (GVfsAfpConnection *afp_connection)
+{
+  GVfsAfpConnectionPrivate *priv;
+  
+  afp_connection->priv = priv =  G_TYPE_INSTANCE_GET_PRIVATE (afp_connection,
+                                                              G_VFS_TYPE_AFP_CONNECTION,
+                                                              GVfsAfpConnectionPrivate);
+
+  priv->addr = NULL;
+  priv->conn = NULL;
+  priv->request_id = 0;
+
+  priv->kRequestQuanta = -1;
+  priv->kServerReplayCacheSize = -1;
+
+  priv->request_queue = g_queue_new ();
+  priv->request_hash = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+                                              NULL, (GDestroyNotify)free_request_data);
+
+  priv->send_loop_running = FALSE;
+  priv->read_loop_running = FALSE;
+}
+
+static void
+g_vfs_afp_connection_finalize (GObject *object)
+{
+  GVfsAfpConnection *afp_connection = (GVfsAfpConnection *)object;
+  GVfsAfpConnectionPrivate *priv = afp_connection->priv;
+
+  if (priv->addr)
+    g_object_unref (priv->addr);
+  
+  if (priv->conn)
+    g_object_unref (priv->conn);
+
+  G_OBJECT_CLASS (g_vfs_afp_connection_parent_class)->finalize (object);
+}
+
+static void
+g_vfs_afp_connection_class_init (GVfsAfpConnectionClass *klass)
+{
+  GObjectClass* object_class = G_OBJECT_CLASS (klass);
+
+  g_type_class_add_private (klass, sizeof (GVfsAfpConnectionPrivate));
+
+  object_class->finalize = g_vfs_afp_connection_finalize;
+
+  signals[ATTENTION] =
+    g_signal_new ("attention",
+                  G_TYPE_FROM_CLASS (object_class),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
+                  0, NULL, NULL, g_cclosure_marshal_VOID__UINT,
+                  G_TYPE_NONE, 1, G_TYPE_UINT);
+}
+
+static void read_reply (GVfsAfpConnection *afp_connection);
+static void send_request (GVfsAfpConnection *afp_connection);
+
+static guint16
+get_request_id (GVfsAfpConnection *afp_connection)
+{
+  GVfsAfpConnectionPrivate *priv = afp_connection->priv;
+
+  return priv->request_id++;
+}
+
 static void
 run_loop (GVfsAfpConnection *afp_connection)
 {
@@ -1302,7 +1406,7 @@ read_reply_sync (GInputStream      *input,
   if (bytes_read < read_count)
   {
     g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                         _("Got EOS"));
+                         _("Connection unexpectedly went down"));
     return FALSE;
   }
 
@@ -1530,13 +1634,25 @@ g_vfs_afp_connection_open (GVfsAfpConnection *afp_connection,
   return TRUE;
 }
 
+GVfsAfpConnection *
+g_vfs_afp_connection_new (GSocketConnectable *addr)
+{
+  GVfsAfpConnection        *afp_connection;
+  GVfsAfpConnectionPrivate *priv;
+
+  afp_connection = g_object_new (G_VFS_TYPE_AFP_CONNECTION, NULL);
+  priv = afp_connection->priv;
+
+  priv->addr = g_object_ref (addr);
+
+  return afp_connection;
+}
+
 GVfsAfpReply *
-g_vfs_afp_connection_get_server_info (GVfsAfpConnection *afp_connection,
-                                      GCancellable *cancellable,
-                                      GError **error)
+g_vfs_afp_query_server_info (GSocketConnectable *addr,
+                             GCancellable *cancellable,
+                             GError **error)
 {
-  GVfsAfpConnectionPrivate *priv = afp_connection->priv;
-  
   GSocketClient *client;
   GIOStream *conn;
   gboolean res;
@@ -1544,7 +1660,7 @@ g_vfs_afp_connection_get_server_info (GVfsAfpConnection *afp_connection,
   char *data;
 
   client = g_socket_client_new ();
-  conn = G_IO_STREAM (g_socket_client_connect (client, priv->addr, cancellable, error));
+  conn = G_IO_STREAM (g_socket_client_connect (client, addr, cancellable, error));
   g_object_unref (client);
 
   if (!conn)
@@ -1571,74 +1687,3 @@ g_vfs_afp_connection_get_server_info (GVfsAfpConnection *afp_connection,
   return g_vfs_afp_reply_new (dsi_header.errorCode, data,
                               dsi_header.totalDataLength, TRUE);
 }
-
-GVfsAfpConnection *
-g_vfs_afp_connection_new (GSocketConnectable *addr)
-{
-  GVfsAfpConnection        *afp_connection;
-  GVfsAfpConnectionPrivate *priv;
-
-  afp_connection = g_object_new (G_VFS_TYPE_AFP_CONNECTION, NULL);
-  priv = afp_connection->priv;
-
-  priv->addr = g_object_ref (addr);
-
-  return afp_connection;
-}
-
-static void
-g_vfs_afp_connection_init (GVfsAfpConnection *afp_connection)
-{
-  GVfsAfpConnectionPrivate *priv;
-  
-  afp_connection->priv = priv =  G_TYPE_INSTANCE_GET_PRIVATE (afp_connection,
-                                                              G_VFS_TYPE_AFP_CONNECTION,
-                                                              GVfsAfpConnectionPrivate);
-
-  priv->addr = NULL;
-  priv->conn = NULL;
-  priv->request_id = 0;
-
-  priv->kRequestQuanta = -1;
-  priv->kServerReplayCacheSize = -1;
-
-  priv->request_queue = g_queue_new ();
-  priv->request_hash = g_hash_table_new_full (g_direct_hash, g_direct_equal,
-                                              NULL, (GDestroyNotify)free_request_data);
-
-  priv->send_loop_running = FALSE;
-  priv->read_loop_running = FALSE;
-}
-
-static void
-g_vfs_afp_connection_finalize (GObject *object)
-{
-  GVfsAfpConnection *afp_connection = (GVfsAfpConnection *)object;
-  GVfsAfpConnectionPrivate *priv = afp_connection->priv;
-
-  if (priv->addr)
-    g_object_unref (priv->addr);
-  
-  if (priv->conn)
-    g_object_unref (priv->conn);
-
-  G_OBJECT_CLASS (g_vfs_afp_connection_parent_class)->finalize (object);
-}
-
-static void
-g_vfs_afp_connection_class_init (GVfsAfpConnectionClass *klass)
-{
-  GObjectClass* object_class = G_OBJECT_CLASS (klass);
-
-  g_type_class_add_private (klass, sizeof (GVfsAfpConnectionPrivate));
-
-  object_class->finalize = g_vfs_afp_connection_finalize;
-
-  signals[ATTENTION] =
-    g_signal_new ("attention",
-                  G_TYPE_FROM_CLASS (object_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
-                  0, NULL, NULL, g_cclosure_marshal_VOID__UINT,
-                  G_TYPE_NONE, 1, G_TYPE_UINT);
-}
-
