@@ -40,6 +40,50 @@ G_DEFINE_TYPE (GVfsAfpServer, g_vfs_afp_server, G_TYPE_OBJECT);
 #define AFP_UAM_DHX       "DHCAST128"
 #define AFP_UAM_DHX2      "DHX2"
 
+GVfsAfpServer *
+g_vfs_afp_server_new (GNetworkAddress *addr)
+{
+  GVfsAfpServer *afp_serv;
+
+  afp_serv = g_object_new (G_VFS_TYPE_AFP_SERVER, NULL);
+
+  afp_serv->addr = addr;
+  afp_serv->conn = g_vfs_afp_connection_new (G_SOCKET_CONNECTABLE (addr));
+  
+  return afp_serv;
+}
+
+static void
+g_vfs_afp_server_init (GVfsAfpServer *afp_serv)
+{
+  afp_serv->machine_type = NULL;
+  afp_serv->server_name = NULL;
+  afp_serv->utf8_server_name = NULL;
+  afp_serv->uams = NULL;
+  afp_serv->version = AFP_VERSION_INVALID;
+}
+
+static void
+g_vfs_afp_server_finalize (GObject *object)
+{
+  GVfsAfpServer *afp_serv = G_VFS_AFP_SERVER (object);
+
+  g_free (afp_serv->machine_type);
+  g_free (afp_serv->server_name);
+  g_free (afp_serv->utf8_server_name);
+  
+  g_slist_free_full (afp_serv->uams, g_free);
+
+  G_OBJECT_CLASS (g_vfs_afp_server_parent_class)->finalize (object);
+}
+
+static void
+g_vfs_afp_server_class_init (GVfsAfpServerClass *klass)
+{
+  GObjectClass* object_class = G_OBJECT_CLASS (klass);
+
+  object_class->finalize = g_vfs_afp_server_finalize;
+}
 
 static const char *
 afp_version_to_string (AfpVersion afp_version)
@@ -123,8 +167,8 @@ dhx2_login (GVfsAfpServer *afp_serv,
   
   if (strlen (password) > 256)
   {
-    g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
-                         _("Server doesn't support passwords longer than 256 characters"));
+    g_set_error (error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
+                 _("Server doesn't support passwords longer than %d characters"), 256);
     goto error;
   }
 
@@ -361,8 +405,7 @@ error:
   goto cleanup;
   
 generic_error:
-  g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-               _("Login to AFP server %s failed"), afp_serv->server_name);
+  g_propagate_error (error, afp_result_code_to_gerror (res_code));
   goto error;
 }
 
@@ -410,8 +453,8 @@ dhx_login (GVfsAfpServer *afp_serv,
 
   if (strlen (password) > 64)
   {
-    g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
-                         _("Server doesn't support passwords longer than 64 characters"));
+    g_set_error(error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
+                _("Server doesn't support passwords longer than %d characters"), 64);
     return FALSE;
   }
   
@@ -581,8 +624,7 @@ error:
   goto done;
   
 generic_error:
-  g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-               _("Login to AFP server %s failed"), afp_serv->server_name);
+  g_propagate_error (error, afp_result_code_to_gerror (res_code));
   res = FALSE;
   goto done;
 }
@@ -641,9 +683,7 @@ do_login (GVfsAfpServer *afp_serv,
           break;
           
         default:
-          g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                       _("Anonymous login to AFP server %s failed, got error code: %d"),
-                       afp_serv->server_name, res_code);
+          g_propagate_error (error, afp_result_code_to_gerror (res_code));
           break;
       }
       
@@ -684,8 +724,8 @@ get_server_info (GVfsAfpServer *afp_serv,
   guint8 count;
   guint i;
 
-  reply = g_vfs_afp_connection_get_server_info (afp_serv->conn, cancellable,
-                                                error);
+  reply = g_vfs_afp_query_server_info (G_SOCKET_CONNECTABLE (afp_serv->addr),
+                                       cancellable, error);
   if (!reply)
     return FALSE;
 
@@ -757,8 +797,128 @@ get_server_info (GVfsAfpServer *afp_serv,
   return TRUE;
 }
 
+static gboolean
+get_server_parms (GVfsAfpServer *server,
+                  GCancellable  *cancellable,
+                  GError       **error)
+{
+  GVfsAfpCommand *comm;
+  GVfsAfpReply   *reply;
+  gboolean        res;
+  AfpResultCode   res_code;
+  gint32          server_time;
+  
+  /* Get Server Parameters */
+  comm = g_vfs_afp_command_new (AFP_COMMAND_GET_SRVR_PARMS);
+  /* pad byte */
+  g_vfs_afp_command_put_byte (comm, 0);
+
+  res = g_vfs_afp_connection_send_command_sync (server->conn, comm, cancellable,
+                                                error);
+  g_object_unref (comm);
+  if (!res)
+    return FALSE;
+
+  reply = g_vfs_afp_connection_read_reply_sync (server->conn, cancellable, error);
+  if (!reply)
+    return FALSE;
+
+  res_code = g_vfs_afp_reply_get_result_code (reply);
+  if (res_code != AFP_RESULT_NO_ERROR)
+  {
+    g_object_unref (reply);
+
+    g_propagate_error (error, afp_result_code_to_gerror (res_code));
+    return FALSE;
+  }
+
+  /* server time */
+  g_vfs_afp_reply_read_int32 (reply, &server_time);
+  server->time_diff = (g_get_real_time () / G_USEC_PER_SEC) - server_time;
+
+  g_object_unref (reply);
+
+  return TRUE;
+}
+
+static gboolean
+get_userinfo (GVfsAfpServer *server,
+              GCancellable  *cancellable,
+              GError       **error)
+{
+  GVfsAfpCommand *comm;
+  guint16 bitmap;
+  gboolean res;
+
+  GVfsAfpReply *reply;
+  AfpResultCode res_code;
+
+  comm = g_vfs_afp_command_new (AFP_COMMAND_GET_USER_INFO);
+  /* Flags, ThisUser = 1 */
+  g_vfs_afp_command_put_byte (comm, 0x01);
+  /* UserId */
+  g_vfs_afp_command_put_int32 (comm, 0);
+  /* Bitmap */
+  bitmap = AFP_GET_USER_INFO_BITMAP_GET_UID_BIT | AFP_GET_USER_INFO_BITMAP_GET_GID_BIT;
+  g_vfs_afp_command_put_uint16 (comm, bitmap);
+
+  res = g_vfs_afp_connection_send_command_sync (server->conn,
+                                                comm, cancellable, error);
+  g_object_unref (comm);
+  if (!res)
+    return FALSE;
+
+  reply = g_vfs_afp_connection_read_reply_sync (server->conn,
+                                                cancellable, error);
+  if (!reply)
+    return FALSE;
+
+  res_code = g_vfs_afp_reply_get_result_code (reply);
+  if (res_code != AFP_RESULT_NO_ERROR)
+  {
+    g_object_unref (reply);
+
+    switch (res_code)
+    {
+      case AFP_RESULT_ACCESS_DENIED:
+        g_set_error (error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
+                     _("Permission denied"));
+        break;
+        break;
+      case AFP_RESULT_CALL_NOT_SUPPORTED:
+        g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+                     _("Command is not supported by server"));
+        break;
+      case AFP_RESULT_PWD_EXPIRED_ERR:
+        g_set_error (error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
+                     _("User's password has expired"));
+        break;
+      case AFP_RESULT_PWD_NEEDS_CHANGE_ERR:
+        g_set_error (error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
+                     _("User's password needs to be changed"));
+        break;
+
+      default:
+        g_propagate_error (error, afp_result_code_to_gerror (res_code));
+        break;
+    }
+    return FALSE;
+  }
+
+  /* Bitmap */
+  g_vfs_afp_reply_read_uint16 (reply, NULL);
+  /* UID */
+  g_vfs_afp_reply_read_uint32 (reply, &server->user_id);
+  /* GID */
+  g_vfs_afp_reply_read_uint32 (reply, &server->group_id);
+
+  g_object_unref (reply);
+  
+  return TRUE;
+}
+
 gboolean
-g_vfs_afp_server_login (GVfsAfpServer *afp_serv,
+g_vfs_afp_server_login (GVfsAfpServer *server,
                         const char     *initial_user,
                         GMountSource   *mount_source,
                         char           **logged_in_user,
@@ -773,7 +933,7 @@ g_vfs_afp_server_login (GVfsAfpServer *afp_serv,
   char *prompt = NULL;
   GError *err = NULL;
 
-  res = get_server_info (afp_serv, cancellable, error);
+  res = get_server_info (server, cancellable, error);
   if (!res)
     return FALSE;
 
@@ -782,7 +942,7 @@ g_vfs_afp_server_login (GVfsAfpServer *afp_serv,
   if (initial_user)
   {
     if (g_str_equal (initial_user, "anonymous") &&
-        g_slist_find_custom (afp_serv->uams, AFP_UAM_NO_USER, (GCompareFunc)g_strcmp0))
+        g_slist_find_custom (server->uams, AFP_UAM_NO_USER, (GCompareFunc)g_strcmp0))
     {
       user = NULL;
       password = NULL;
@@ -791,12 +951,12 @@ g_vfs_afp_server_login (GVfsAfpServer *afp_serv,
     }
 
     else if (g_vfs_keyring_lookup_password (initial_user,
-                                            g_network_address_get_hostname (afp_serv->addr),
+                                            g_network_address_get_hostname (server->addr),
                                             NULL,
                                             "afp",
                                             NULL,
                                             NULL,
-                                            g_network_address_get_port (afp_serv->addr),
+                                            g_network_address_get_port (server->addr),
                                             &user,
                                             NULL,
                                             &password) &&
@@ -827,10 +987,10 @@ g_vfs_afp_server_login (GVfsAfpServer *afp_serv,
     /* create prompt */
     if (initial_user)
       /* Translators: the first %s is the username, the second the host name */
-      g_string_append_printf (str, _("Enter password for afp as %s on %s"), initial_user, afp_serv->server_name);
+      g_string_append_printf (str, _("Enter password for afp as %s on %s"), initial_user, server->server_name);
     else
       /* translators: %s here is the hostname */
-      g_string_append_printf (str, _("Enter password for afp on %s"), afp_serv->server_name);
+      g_string_append_printf (str, _("Enter password for afp on %s"), server->server_name);
 
     prompt = g_string_free (str, FALSE);
 
@@ -840,7 +1000,7 @@ g_vfs_afp_server_login (GVfsAfpServer *afp_serv,
     {
       flags |= G_ASK_PASSWORD_NEED_USERNAME;
       
-      if (g_slist_find_custom (afp_serv->uams, AFP_UAM_NO_USER, (GCompareFunc)g_strcmp0))
+      if (g_slist_find_custom (server->uams, AFP_UAM_NO_USER, (GCompareFunc)g_strcmp0))
         flags |= G_ASK_PASSWORD_ANONYMOUS_SUPPORTED;
     }
 
@@ -873,15 +1033,15 @@ g_vfs_afp_server_login (GVfsAfpServer *afp_serv,
 try_login:
 
     /* Open connection */
-    res = g_vfs_afp_connection_open (afp_serv->conn, cancellable, &err);
+    res = g_vfs_afp_connection_open (server->conn, cancellable, &err);
     if (!res)
       break;
 
-    res = do_login (afp_serv, user, password, anonymous,
+    res = do_login (server, user, password, anonymous,
                     cancellable, &err);
     if (!res)
     {
-      g_vfs_afp_connection_close (afp_serv->conn, cancellable, NULL);
+      g_vfs_afp_connection_close (server->conn, cancellable, NULL);
 
       if (!g_error_matches (err, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED))
         break;
@@ -898,29 +1058,31 @@ try_login:
   g_free (olduser);
 
   if (!res)
-  {
-    g_free (user);
-    g_free (password);
+    goto error;
 
-    g_propagate_error (error, err);
-    return FALSE;
-  }
+    /* Get server parms */
+  if (!get_server_parms (server, cancellable, &err))
+    goto error;
 
+  /* Get user info */
+  if (!get_userinfo (server, cancellable, &err))
+    goto error;
+  
   if (prompt && !anonymous)
   {
     /* a prompt was created, so we have to save the password */
     g_vfs_keyring_save_password (user,
-                                 g_network_address_get_hostname (afp_serv->addr),
+                                 g_network_address_get_hostname (server->addr),
                                  NULL,
                                  "afp",
                                  NULL,
                                  NULL,
-                                 g_network_address_get_port (afp_serv->addr),
+                                 g_network_address_get_port (server->addr),
                                  password,
                                  password_save);
     g_free (prompt);
   }
-
+  
   if (logged_in_user)
   {
     if (anonymous)
@@ -934,50 +1096,392 @@ try_login:
   g_free (password);
 
   return TRUE;
+
+error:
+  g_free (user);
+  g_free (password);
+  g_propagate_error (error, err);
+  return FALSE;
 }
 
-GVfsAfpServer *
-g_vfs_afp_server_new (GNetworkAddress *addr)
+/*
+ * g_vfs_server_time_to_local_time:
+ * 
+ * @server: a #GVfsAfpServer
+ * @server_time: a time value in server time
+ * 
+ * Returns: the time converted to local time
+ */
+gint64
+g_vfs_afp_server_time_to_local_time (GVfsAfpServer *server,
+                                     gint32         server_time)
 {
-  GVfsAfpServer *afp_serv;
+  return server_time + server->time_diff;
+}
 
-  afp_serv = g_object_new (G_VFS_TYPE_AFP_SERVER, NULL);
 
-  afp_serv->addr = addr;
-  afp_serv->conn = g_vfs_afp_connection_new (G_SOCKET_CONNECTABLE (addr));
+static void
+volume_data_free (GVfsAfpVolumeData *vol_data)
+{
+  g_free (vol_data->name);
+  g_slice_free (GVfsAfpVolumeData, vol_data);
+}
+
+
+static void
+get_volumes_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GVfsAfpConnection *afp_conn = G_VFS_AFP_CONNECTION (source_object);
+  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (user_data);
+
+  GVfsAfpReply *reply;
+  GError *err = NULL;
+  AfpResultCode res_code;
   
-  return afp_serv;
-}
-
-static void
-g_vfs_afp_server_init (GVfsAfpServer *afp_serv)
-{
-  afp_serv->machine_type = NULL;
-  afp_serv->server_name = NULL;
-  afp_serv->utf8_server_name = NULL;
-  afp_serv->uams = NULL;
-  afp_serv->version = AFP_VERSION_INVALID;
-}
-
-static void
-g_vfs_afp_server_finalize (GObject *object)
-{
-  GVfsAfpServer *afp_serv = G_VFS_AFP_SERVER (object);
-
-  g_free (afp_serv->machine_type);
-  g_free (afp_serv->server_name);
-  g_free (afp_serv->utf8_server_name);
+  guint8 num_volumes, i;
+  GPtrArray *volumes;
   
-  g_slist_free_full (afp_serv->uams, g_free);
+  reply = g_vfs_afp_connection_send_command_finish (afp_conn, res, &err);
+  if (!reply)
+  {
+    g_simple_async_result_take_error (simple, err);
+    goto done;
+  }
 
-  G_OBJECT_CLASS (g_vfs_afp_server_parent_class)->finalize (object);
+  res_code = g_vfs_afp_reply_get_result_code (reply);
+  if (res_code != AFP_RESULT_NO_ERROR)
+  {
+    g_object_unref (reply);
+
+    g_simple_async_result_take_error (simple, afp_result_code_to_gerror (res_code));
+    goto done;
+  }
+  
+  /* server time */
+  g_vfs_afp_reply_read_int32 (reply, NULL);
+
+  /* NumVolStructures */
+  g_vfs_afp_reply_read_byte (reply, &num_volumes);
+  
+  volumes = g_ptr_array_sized_new (num_volumes);
+  g_ptr_array_set_free_func (volumes, (GDestroyNotify)volume_data_free);
+  for (i = 0; i < num_volumes; i++)
+  {
+    guint8 flags;
+    char *vol_name;
+
+    GVfsAfpVolumeData *volume_data;
+
+    g_vfs_afp_reply_read_byte (reply, &flags);
+    g_vfs_afp_reply_read_pascal (reply, &vol_name);
+    if (!vol_name)
+      continue;
+
+    volume_data = g_slice_new (GVfsAfpVolumeData);
+    volume_data->flags = flags;
+    volume_data->name = vol_name;
+
+    g_ptr_array_add (volumes, volume_data);
+  }
+  g_object_unref (reply);
+
+  g_simple_async_result_set_op_res_gpointer (simple, volumes,
+                                             (GDestroyNotify)g_ptr_array_unref);
+done:
+  g_simple_async_result_complete (simple);
+  g_object_unref (simple);
+}
+
+/*
+ * g_vfs_afp_server_get_volumes:
+ * 
+ * @server: a #GVfsAfpServer
+ * @cancellable: optional #GCancellable object, %NULL to ignore.
+ * @callback: callback to call when the request is satisfied.
+ * @user_data: the data to pass to callback function.
+ * 
+ * Asynchronously retrieves the volumes available on @server.
+ */
+void
+g_vfs_afp_server_get_volumes (GVfsAfpServer       *server,
+                              GCancellable        *cancellable,
+                              GAsyncReadyCallback  callback,
+                              gpointer             user_data)
+{
+  GVfsAfpCommand *comm;
+  GSimpleAsyncResult *simple;
+  
+  /* Get Server Parameters */
+  comm = g_vfs_afp_command_new (AFP_COMMAND_GET_SRVR_PARMS);
+  /* pad byte */
+  g_vfs_afp_command_put_byte (comm, 0);
+
+  simple = g_simple_async_result_new (G_OBJECT (server), callback, user_data,
+                                      g_vfs_afp_server_get_volumes);
+  
+  g_vfs_afp_connection_send_command (server->conn, comm, NULL, get_volumes_cb,
+                                     cancellable, simple);
+}
+
+/*
+ * g_vfs_afp_server_get_volumes_finish:
+ * 
+ * @server: a #GVfsAfpServer.
+ * @result: a #GAsyncResult.
+ * @error: a #GError, %NULL to ignore.
+ * 
+ * Finalizes the asynchronous operation started by
+ * g_vfs_afp_server_get_volumes.
+ * 
+ * Returns: A #GPtrArray containing the volumes #GVfsAfpVolumeData structures or
+ * %NULL on error.
+ *                      
+ */
+GPtrArray *
+g_vfs_afp_server_get_volumes_finish (GVfsAfpServer  *server,
+                                     GAsyncResult   *result,
+                                     GError         **error)
+{
+  GSimpleAsyncResult *simple;
+  
+  g_return_val_if_fail (g_simple_async_result_is_valid (result,
+                                                        G_OBJECT (server),
+                                                        g_vfs_afp_server_get_volumes),
+                        NULL);
+
+  simple = (GSimpleAsyncResult *)result;
+
+  if (g_simple_async_result_propagate_error (simple, error))
+    return NULL;
+
+  return g_ptr_array_ref ((GPtrArray *)g_simple_async_result_get_op_res_gpointer (simple));
+}
+
+GVfsAfpVolume *
+g_vfs_afp_server_mount_volume_sync (GVfsAfpServer *server,
+                                    const char    *volume_name,
+                                    GCancellable  *cancellable,
+                                    GError **error)
+{
+  GVfsAfpVolume *volume;
+
+  volume = g_vfs_afp_volume_new (server);
+  if (!g_vfs_afp_volume_mount_sync (volume, volume_name, cancellable, error))
+  {
+    g_object_unref (volume);
+    return NULL;
+  }
+
+  return volume;
 }
 
 static void
-g_vfs_afp_server_class_init (GVfsAfpServerClass *klass)
+set_access_attributes_trusted (GFileInfo *info,
+                               guint32 perm)
 {
-  GObjectClass* object_class = G_OBJECT_CLASS (klass);
-
-  object_class->finalize = g_vfs_afp_server_finalize;
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ,
+				     perm & 0x4);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE,
+				     perm & 0x2);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE,
+				     perm & 0x1);
 }
 
+/* For files we don't own we can't trust a negative response to this check, as
+   something else could allow us to do the operation, for instance an ACL
+   or some sticky bit thing */
+static void
+set_access_attributes (GFileInfo *info,
+                       guint32 perm)
+{
+  if (perm & 0x4)
+    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ,
+				       TRUE);
+  if (perm & 0x2)
+    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE,
+				       TRUE);
+  if (perm & 0x1)
+    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE,
+				       TRUE);
+}
+
+void
+g_vfs_afp_server_fill_info (GVfsAfpServer *server,
+                            GFileInfo     *info,
+                            GVfsAfpReply  *reply,
+                            gboolean       directory,
+                            guint16        bitmap)
+{
+  goffset start_pos;
+
+  if (directory)
+  {
+    GIcon *icon;
+    
+    g_file_info_set_file_type (info, G_FILE_TYPE_DIRECTORY);
+    g_file_info_set_content_type (info, "inode/directory");
+
+    icon = g_themed_icon_new ("folder");
+    g_file_info_set_icon (info, icon);
+    g_object_unref (icon);
+  }
+  else
+    g_file_info_set_file_type (info, G_FILE_TYPE_REGULAR);
+
+  
+  start_pos = g_vfs_afp_reply_get_pos (reply);
+
+  if (bitmap & AFP_FILEDIR_BITMAP_ATTRIBUTE_BIT)
+  {
+    guint16 attributes;
+
+    g_vfs_afp_reply_read_uint16 (reply, &attributes);
+    
+    if (attributes & AFP_FILEDIR_ATTRIBUTES_BITMAP_INVISIBLE_BIT)
+      g_file_info_set_is_hidden (info, TRUE);
+  }
+
+  if (bitmap & AFP_FILEDIR_BITMAP_PARENT_DIR_ID_BIT)
+  {
+    guint32 parent_dir_id;
+
+    g_vfs_afp_reply_read_uint32 (reply, &parent_dir_id);
+    g_file_info_set_attribute_uint32 (info, "afp::parent-dir-id", parent_dir_id);
+  }
+  
+  if (bitmap & AFP_FILEDIR_BITMAP_CREATE_DATE_BIT)
+  {
+    gint32 create_date;
+    gint64 create_date_local;
+
+    g_vfs_afp_reply_read_int32 (reply, &create_date);
+    
+    create_date_local = g_vfs_afp_server_time_to_local_time (server, create_date);
+    g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_CREATED,
+                                      create_date_local);
+  }
+
+  if (bitmap & AFP_FILEDIR_BITMAP_MOD_DATE_BIT)
+  {
+    gint32 mod_date;
+    guint64 mod_date_unix;
+    char *etag;
+
+    g_vfs_afp_reply_read_int32 (reply, &mod_date);
+    mod_date_unix = g_vfs_afp_server_time_to_local_time (server, mod_date);
+    
+    g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED,
+                                      mod_date_unix);
+
+    etag = g_strdup_printf ("%"G_GUINT64_FORMAT, mod_date_unix);
+    g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_ETAG_VALUE, etag);
+    g_free (etag);
+  }
+
+  if (bitmap & AFP_FILEDIR_BITMAP_NODE_ID_BIT)
+  {
+    guint32 node_id;
+
+    g_vfs_afp_reply_read_uint32 (reply, &node_id);
+    g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_AFP_NODE_ID, node_id);
+  }
+  
+  /* Directory specific attributes */
+  if (directory)
+  {
+    if (bitmap & AFP_DIR_BITMAP_OFFSPRING_COUNT_BIT)
+    {
+      guint16 offspring_count;
+
+      g_vfs_afp_reply_read_uint16 (reply, &offspring_count);
+      g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_AFP_CHILDREN_COUNT,
+                                        offspring_count);
+    }
+  }
+  
+  /* File specific attributes */
+  else
+  {
+    if (bitmap & AFP_FILE_BITMAP_EXT_DATA_FORK_LEN_BIT)
+    {
+      guint64 fork_len;
+
+      g_vfs_afp_reply_read_uint64 (reply, &fork_len);
+      g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_STANDARD_SIZE,
+                                        fork_len);
+    }
+  }
+  
+  if (bitmap & AFP_FILEDIR_BITMAP_UTF8_NAME_BIT)
+  {
+    guint16 UTF8Name_offset;
+    goffset old_pos;
+    GVfsAfpName *afp_name;
+    char *utf8_name;
+
+    g_vfs_afp_reply_read_uint16 (reply, &UTF8Name_offset);
+    /* Pad */
+    g_vfs_afp_reply_read_uint32 (reply, NULL);
+
+    old_pos = g_vfs_afp_reply_get_pos (reply);
+    g_vfs_afp_reply_seek (reply, start_pos + UTF8Name_offset, G_SEEK_SET);
+
+    g_vfs_afp_reply_read_afp_name (reply, TRUE, &afp_name);
+    utf8_name = g_vfs_afp_name_get_string (afp_name);    
+    g_vfs_afp_name_unref (afp_name);
+
+    g_file_info_set_name (info, utf8_name);
+    g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
+                                      utf8_name);
+
+    /* Set file as hidden if it begins with a dot */
+    if (utf8_name[0] == '.')
+      g_file_info_set_is_hidden (info, TRUE);
+
+    if (!directory)
+    {
+      char *content_type;
+      GIcon *icon;
+
+      content_type = g_content_type_guess (utf8_name, NULL, 0, NULL);
+      g_file_info_set_content_type (info, content_type);
+      g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE,
+                                        content_type);
+
+      icon = g_content_type_get_icon (content_type);
+      g_file_info_set_icon (info, icon);
+
+      g_object_unref (icon);
+      g_free (content_type);
+    }
+    
+    g_free (utf8_name);
+
+    g_vfs_afp_reply_seek (reply, old_pos, G_SEEK_SET);
+  }
+
+  if (bitmap & AFP_FILEDIR_BITMAP_UNIX_PRIVS_BIT)
+  {
+    guint32 uid, gid, permissions, ua_permissions;
+
+    g_vfs_afp_reply_read_uint32 (reply, &uid);
+    g_vfs_afp_reply_read_uint32 (reply, &gid);
+    g_vfs_afp_reply_read_uint32 (reply, &permissions);
+    /* ua_permissions */
+    g_vfs_afp_reply_read_uint32 (reply, &ua_permissions);
+
+    g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_MODE, permissions);
+    g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_UID, uid);
+    g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_GID, gid);
+
+    g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_AFP_UA_PERMISSIONS,
+                                      ua_permissions);
+    
+    if (uid == server->user_id)
+      set_access_attributes_trusted (info, (permissions >> 6) & 0x7);
+    else if (gid == server->group_id)
+      set_access_attributes (info, (permissions >> 3) & 0x7);
+    else
+      set_access_attributes (info, (permissions >> 0) & 0x7);
+  }
+}
\ No newline at end of file
